@@ -30,6 +30,7 @@ static GPdfURIInput *uri_in;
 /* The list of all open windows */
 static GList *window_list = NULL;
 
+static void gpdf_window_update_window_title (GPdfWindow *);
 
 static gboolean
 create_window (gpointer data)
@@ -103,11 +104,11 @@ open_request_handler (GPdfURIInput *in, const char *uri, gpointer user_data)
 
 	window = find_empty_or_create_new_window (window_list);
 	if (gpdf_window_open (window, uri)) {
+	  	gpdf_window_update_window_title (window);
 		gtk_widget_show_now (GTK_WIDGET (window));
-	} else {
-		if (g_list_length (window_list) > 1)
-			gtk_widget_destroy (GTK_WIDGET (window));
-	}
+	} else
+	  if (g_list_length (window_list) > 1)
+	    	gtk_widget_destroy (GTK_WIDGET (window));
 }
 
 #ifdef PDF_DEBUG
@@ -118,12 +119,6 @@ open_request_logger (GPdfURIInput *in, const char *uri, gpointer user_data)
 }
 #endif
 
-static void
-set_window_icon_default (void)
-{
-	gtk_window_set_default_icon_from_file (GNOMEICONDIR "/gnome-pdf.png", NULL);
-}
-
 int
 main (int argc, char **argv)
 {
@@ -146,7 +141,7 @@ main (int argc, char **argv)
 	if (bonobo_ui_init ("GNOME PDF Viewer", VERSION, &argc, argv) == FALSE)
 		g_error (_("Could not initialize Bonobo!\n"));
 
-	set_window_icon_default ();
+	gtk_window_set_default_icon_from_file (GNOMEICONDIR "/gnome-pdf.png", NULL);
 
 	client = gnome_master_client ();
 	
@@ -156,7 +151,7 @@ main (int argc, char **argv)
 	ctx = g_value_get_pointer (&value);
 	g_value_unset (&value);
 
-	gtk_idle_add (handle_cmdline_args, ctx);
+	g_idle_add (handle_cmdline_args, ctx);
 
 	uri_in = g_object_new (GPDF_TYPE_URI_INPUT, NULL);
 	recent_facade = g_object_new (GPDF_TYPE_RECENT_FACADE, NULL);
@@ -192,6 +187,8 @@ struct _GPdfWindowPrivate {
 	GtkWidget *slot;
 
 	EggRecentView *recent_view;
+
+	GtkWidget *exit_fullscreen_popup;
 };
 
 /* Drag target types */
@@ -234,13 +231,7 @@ gw_add_control_to_ui (GPdfWindow *gpdf_window)
 
 	g_object_ref (G_OBJECT (bonobo_widget));
 	priv->bonobo_widget = BONOBO_WIDGET (bonobo_widget);
-
-	bonobo_control_frame_control_activate (
-		bonobo_widget_get_control_frame (BONOBO_WIDGET (bonobo_widget)));
-
-	gtk_widget_show (bonobo_widget);
 	gtk_container_add (GTK_CONTAINER (priv->slot), bonobo_widget);
-
 	CORBA_exception_free (&ev);
 
 	return TRUE;
@@ -306,8 +297,7 @@ gw_control_load_pdf (GPdfWindow *gpdf_window, const char *uri)
 		control, "IDL:Bonobo/PersistFile:1.0", &ev);
 
 	if (BONOBO_EX (&ev) || persist == CORBA_OBJECT_NIL) {
-		gnome_error_dialog (
-			"Panic: component doesn't implement PersistFile.");
+		g_warning ("Panic: component doesn't implement PersistFile.");
 		goto error_persist;
 	}
 
@@ -316,9 +306,6 @@ gw_control_load_pdf (GPdfWindow *gpdf_window, const char *uri)
 	if (BONOBO_EX (&ev))
 		goto error_stream;
 
-	/* FIXME doesn't belong here */
-	gpdf_window_update_window_title (gpdf_window);
-
 	success = TRUE;		
 
 error_stream:
@@ -332,13 +319,37 @@ error_persist:
 gboolean
 gpdf_window_open (GPdfWindow *gpdf_window, const char *name)
 {
+	GPdfWindowPrivate *priv;
+
 	g_return_val_if_fail (GPDF_IS_NON_NULL_WINDOW (gpdf_window), FALSE);
 	g_return_val_if_fail (!gpdf_window_has_contents (gpdf_window), FALSE);
 
+	priv = gpdf_window->priv;
+
 	if (gw_add_control_to_ui (gpdf_window) == FALSE)
 		return FALSE;
 
-	return gw_control_load_pdf (gpdf_window, name);
+	if (gw_control_load_pdf (gpdf_window, name) == FALSE) {
+	  	gtk_container_remove (GTK_CONTAINER (priv->slot),
+				      GTK_WIDGET (priv->bonobo_widget));
+		gtk_widget_destroy (GTK_WIDGET (priv->bonobo_widget));
+		priv->bonobo_widget = NULL;
+		bonobo_ui_component_set_prop (gpdf_window->priv->ui_component,
+					      "/commands/ViewFullScreen",
+					      "sensitive", "0", NULL);
+		return FALSE;
+	}
+
+	/* Activate component */
+	bonobo_control_frame_control_activate (
+	       bonobo_widget_get_control_frame (priv->bonobo_widget));
+	gtk_widget_show (GTK_WIDGET (priv->bonobo_widget));
+
+	bonobo_ui_component_set_prop (gpdf_window->priv->ui_component,
+				      "/commands/ViewFullScreen",
+				      "sensitive", "1", NULL);
+	
+	return TRUE;
 }
 
 static void
@@ -371,55 +382,177 @@ gw_close_all (void)
 }
 
 static void
-file_dialog_ok (GtkWidget *widget, gboolean *dialog_result)
+gpdf_window_set_fullscreen (GPdfWindow *gpdf_window, gboolean fullscreen)
 {
-	GtkFileSelection *fsel;
-	const gchar *file_name;
+	if (fullscreen)
+		gtk_window_fullscreen (GTK_WINDOW (gpdf_window));
+	else
+		gtk_window_unfullscreen (GTK_WINDOW (gpdf_window));
+}
 
-	fsel = GTK_FILE_SELECTION (
-		gtk_widget_get_ancestor (widget, GTK_TYPE_FILE_SELECTION));
-	file_name = gtk_file_selection_get_filename (fsel);
+static void
+exit_fullscreen_button_clicked_cb (GtkWidget *button, GPdfWindow *window)
+{
+	gpdf_window_set_fullscreen (window, FALSE);
+}
+
+static void
+update_exit_fullscreen_popup_position (GPdfWindow *gpdf_window)
+{
+	GdkRectangle screen_rect;
+	int popup_height;
+
+	if (gpdf_window->priv->exit_fullscreen_popup == NULL)
+		return;
+
+	gtk_window_get_size (GTK_WINDOW (gpdf_window->priv->exit_fullscreen_popup),
+                             NULL, &popup_height);
+
+	gdk_screen_get_monitor_geometry (
+		gdk_screen_get_default (),
+		gdk_screen_get_monitor_at_window (gdk_screen_get_default (),
+						  GTK_WIDGET (gpdf_window)->window),
+		&screen_rect);
+
+	gtk_window_move (GTK_WINDOW (gpdf_window->priv->exit_fullscreen_popup),
+			 screen_rect.x, screen_rect.height - popup_height);
+}
+
+static void
+screen_size_changed_cb (GdkScreen *screen, GPdfWindow *gpdf_window)
+{
+	update_exit_fullscreen_popup_position (gpdf_window);
+}
+
+static void
+gpdf_window_init_fullscreen_popup (GPdfWindow *gpdf_window)
+{
+	GtkWidget *popup, *button, *hbox, *icon, *label;
+	GtkAccelGroup *accel_group;
+
+	accel_group = gtk_accel_group_new ();
+	
+	popup = gtk_window_new (GTK_WINDOW_POPUP);
+	gpdf_window->priv->exit_fullscreen_popup = popup;
+
+	button = gtk_button_new ();
+	g_signal_connect (button, "clicked",
+			  G_CALLBACK (exit_fullscreen_button_clicked_cb),
+			  gpdf_window);
+	gtk_widget_add_accelerator (button, "clicked", accel_group,
+				    GDK_Escape, 0,
+				    GTK_ACCEL_VISIBLE);
+	gtk_widget_add_accelerator (button, "clicked", accel_group,
+				    GDK_F11, 0,
+				    GTK_ACCEL_VISIBLE);
+	gtk_widget_show (button);
+	gtk_container_add (GTK_CONTAINER (popup), button);
+
+	hbox = gtk_hbox_new (FALSE, 2);
+	gtk_widget_show (hbox);
+	gtk_container_add (GTK_CONTAINER (button), hbox);
+
+	icon = gtk_image_new_from_stock (GTK_STOCK_QUIT, GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (icon);
+	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
+
+	label = gtk_label_new (_("Exit Full Screen"));
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+	update_exit_fullscreen_popup_position (gpdf_window);
+
+	gtk_widget_show (popup);
+
+	g_signal_connect (G_OBJECT (gdk_screen_get_default ()),
+                          "size-changed", G_CALLBACK (screen_size_changed_cb),
+			  gpdf_window);
+
+	gtk_window_add_accel_group (GTK_WINDOW (gpdf_window), accel_group);
+}
+
+static gboolean
+gpdf_window_window_state_changed (GtkWidget *widget, GdkEventWindowState *event,
+				  gpointer user_data)
+{
+	gboolean fullscreen;
+	GPdfWindow *gpdf_window;
+	BonoboUIComponent *uic;
+
+	if (!(event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN))
+		return FALSE;
+
+	g_return_val_if_fail (GPDF_IS_NON_NULL_WINDOW (widget), FALSE);
+
+	gpdf_window = GPDF_WINDOW (widget);
+	fullscreen = event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN;
+
+	uic = gpdf_window->priv->ui_component;
+	if (uic == NULL)
+		return FALSE;
+
+	bonobo_ui_component_set_prop (uic, "/commands/ViewFullScreen", "state",
+				      fullscreen ? "1" : "0", NULL);
+
+	if (fullscreen) {
+		bonobo_ui_component_freeze (uic, NULL);
+		bonobo_ui_component_set_prop (uic,
+					      "/menu", "hidden", "1", NULL);
+		bonobo_ui_component_set_prop (uic,
+					      "/Toolbar", "hidden", "1", NULL);
+		bonobo_ui_component_thaw (uic, NULL);
+
+		gpdf_window_init_fullscreen_popup (gpdf_window);
+	} else {
+		bonobo_ui_component_freeze (uic, NULL);
+		bonobo_ui_component_set_prop (uic,
+					      "/menu", "hidden", "0", NULL);
+		bonobo_ui_component_set_prop (uic,
+					      "/Toolbar", "hidden", "0", NULL);
+		bonobo_ui_component_thaw (uic, NULL);
+
+		gtk_widget_destroy (gpdf_window->priv->exit_fullscreen_popup);
+		gpdf_window->priv->exit_fullscreen_popup = NULL;
+        }
+
+	return FALSE;
+}
+
+static void
+file_dialog_response (GtkWidget *chooser, gint response, gboolean *dialog_result)
+{
+	gchar *file_name;
+
+	if (response != GTK_RESPONSE_OK) {
+		gtk_widget_hide (chooser);
+		gtk_main_quit ();
+		return;
+	}
+
+	file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
 
 	if (!g_file_test (file_name, G_FILE_TEST_EXISTS)) {
-		const gchar *pattern;
-
 		g_message ("!file exists in file_dialog_ok: %s", file_name);
-
-		if (fsel->selection_entry == NULL)
-			return;
-
-		pattern = 
-			gtk_entry_get_text (GTK_ENTRY (fsel->selection_entry));
-		gtk_file_selection_complete (fsel, pattern);
-
 	} else if (g_file_test (file_name, G_FILE_TEST_IS_DIR)) {
 		/* Change into directory if that's what the user selected */
-		gint name_len;
 		gchar *dir_name;
 
-		name_len = strlen (file_name);
-		if (name_len < 1 || file_name [name_len - 1] != '/') {
-			/* Needed for the file selector */
-			dir_name = g_strconcat (file_name, "/", NULL);
+		if (strlen (file_name) < 1) {
+			dir_name = g_strdup ("/");
 		} else {
 			dir_name = g_strdup (file_name);
 		}
 
-		gtk_file_selection_set_filename (fsel, dir_name);
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
+						     dir_name);
 		g_free (dir_name);
 	} else {
-		gtk_widget_hide (GTK_WIDGET (fsel));
+		gtk_widget_hide (chooser);
 		gtk_main_quit ();
 		*dialog_result = TRUE;
 	}
-}
 
-static void
-file_dialog_cancel (GtkWidget *widget, gpointer user_data)
-{
-	gtk_widget_hide (
-		gtk_widget_get_ancestor (widget, GTK_TYPE_FILE_SELECTION));
-	gtk_main_quit ();
+	g_free (file_name);
 }
 
 static guint
@@ -431,41 +564,60 @@ file_dialog_delete_event (GtkWidget *widget, GdkEventAny *event)
 }
 
 static gboolean
-gw_ask_for_uri (GPdfWindow *gpdf_window, gchar **uri)
+gw_ask_for_uri (GPdfWindow *gpdf_window, gchar ***uri)
 {
-	GtkFileSelection *fsel;
+	GtkFileChooser *chooser;
 	gboolean accepted = FALSE;
 
 	g_return_val_if_fail (GPDF_IS_NON_NULL_WINDOW (gpdf_window), FALSE);
+	g_return_val_if_fail (uri, FALSE);
 	g_return_val_if_fail (*uri == NULL, FALSE);
 
-	fsel = GTK_FILE_SELECTION (gtk_file_selection_new (_("Load file")));
-	gtk_window_set_modal (GTK_WINDOW (fsel), TRUE);
-	gtk_window_set_transient_for (GTK_WINDOW (fsel),
-				      GTK_WINDOW (gpdf_window));
+	chooser = GTK_FILE_CHOOSER (gtk_file_chooser_dialog_new (_("Load file"),
+								 GTK_WINDOW (gpdf_window),
+								 GTK_FILE_CHOOSER_ACTION_OPEN,
+								 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+								 GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+								 NULL));
+	gtk_file_chooser_set_select_multiple (chooser, TRUE); 
+	gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
 
 	/* Connect the signals for Ok and Cancel */
-	g_signal_connect (G_OBJECT (fsel->ok_button), "clicked",
-			  G_CALLBACK (file_dialog_ok), &accepted);
-	g_signal_connect (G_OBJECT (fsel->cancel_button), "clicked",
-			  G_CALLBACK (file_dialog_cancel), NULL);
+	g_signal_connect (G_OBJECT (chooser), "response",
+			  G_CALLBACK (file_dialog_response), &accepted);
 
-	gtk_window_set_position (GTK_WINDOW (fsel), GTK_WIN_POS_MOUSE);
+	gtk_window_set_position (GTK_WINDOW (chooser), GTK_WIN_POS_MOUSE);
 
 	/* Make sure that we quit the main loop if the window is destroyed */
-	g_signal_connect (G_OBJECT (fsel), "delete_event",
+	g_signal_connect (G_OBJECT (chooser), "delete_event",
 			  G_CALLBACK (file_dialog_delete_event), NULL);
 
 	/* Run the dialog */
-	gtk_widget_show (GTK_WIDGET (fsel));
-	gtk_grab_add (GTK_WIDGET (fsel));
+	gtk_widget_show (GTK_WIDGET (chooser));
+	gtk_grab_add (GTK_WIDGET (chooser));
 	gtk_main ();
 
-	if (accepted)
-		*uri = gnome_vfs_get_uri_from_local_path (
-			gtk_file_selection_get_filename (fsel));
-	
-	gtk_widget_destroy (GTK_WIDGET (fsel));
+	if (accepted) {
+		GSList *files, *iter;
+		guint n; 
+
+		files = gtk_file_chooser_get_filenames (chooser);
+
+		*uri = g_new (gchar*, g_slist_length (files) + 1);
+
+		n = 0;
+		for (iter = files; iter != NULL; iter = iter->next) {
+			(*uri)[n] =
+				gnome_vfs_get_uri_from_local_path (iter->data);
+			g_free (iter->data);
+			n++;
+		}
+		(*uri)[n] = NULL; 
+
+		g_slist_free (files);
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (chooser));
 
 	return accepted;
 }
@@ -473,16 +625,21 @@ gw_ask_for_uri (GPdfWindow *gpdf_window, gchar **uri)
 static void
 gw_open_dialog (GPdfWindow *gpdf_window)
 {
-	gchar *uri = NULL;
+	gchar **uri = NULL;
 
 	g_return_if_fail (GPDF_IS_NON_NULL_WINDOW (gpdf_window));
 
 	if (gw_ask_for_uri (gpdf_window, &uri)) {
+		guint n;
+		
 		g_assert (uri != NULL);
 
-		gpdf_uri_input_open_uri (uri_in, uri);
-
-		g_free (uri);
+		n = 0; 
+		while (uri[n]) {
+			gpdf_uri_input_open_uri (uri_in, uri[n]);
+			n++;
+		}
+		g_strfreev (uri); 
 	}
 }
 
@@ -522,6 +679,23 @@ verb_FileExit_cb (BonoboUIComponent *uic, gpointer user_data,
 	gw_close_all ();
 }
 
+static void
+listener_ViewFullScreen (BonoboUIComponent *uic, const char *path,
+                         Bonobo_UIComponent_EventType type, const char *state,
+                         gpointer user_data)
+{
+	g_return_if_fail (GPDF_IS_NON_NULL_WINDOW (user_data));
+
+	if(type != Bonobo_UIComponent_STATE_CHANGED)
+		return;
+
+	if(!state)
+		return;
+
+	gpdf_window_set_fullscreen(GPDF_WINDOW (user_data), atoi (state));
+}
+
+
 /* Brings attention to a window by raising it and giving it focus */
 static void
 raise_and_focus (GtkWidget *widget)
@@ -531,6 +705,15 @@ raise_and_focus (GtkWidget *widget)
 	gtk_widget_grab_focus (widget);
 }
 
+static void
+verb_HelpContents_cb (BonoboUIComponent *uic, gpointer user_data,
+		     const char *cname)
+{
+	gnome_help_display ("gpdf.xml",
+			    NULL,
+			    NULL);
+}
+
 static void
 verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data,
 		   const char *cname)
@@ -551,6 +734,12 @@ verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data,
 		NULL
 	};
 
+	static const gchar *documenters[] = {
+		N_("Chee Bin HOH <cbhoh@mimos.my>"),
+		N_("Breda McColgan <BredaMcColgan@sun.com>"),
+		NULL
+	};
+
 	static const gchar *translators;
 
 	if (!about) {
@@ -573,7 +762,7 @@ verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data,
 			tmp_str,
 			_("A PDF viewer based on Xpdf"),
 			authors,
-			NULL,
+			documenters,
 			(strcmp (translators,
 				 "translator_credits-PLEASE_ADD_YOURSELF_HERE")
 			 ? translators : NULL),
@@ -692,6 +881,7 @@ static BonoboUIVerb gw_verbs [] = {
 	BONOBO_UI_VERB ("FileClose", verb_FileClose_cb),
 	BONOBO_UI_VERB ("FileExit",  verb_FileExit_cb),
 
+	BONOBO_UI_VERB ("HelpContents", verb_HelpContents_cb),
 	BONOBO_UI_VERB ("HelpAbout", verb_HelpAbout_cb),
 
 	BONOBO_UI_VERB_END
@@ -718,6 +908,9 @@ gw_setup_toplevel_ui (GPdfWindow *gpdf_window)
 	bonobo_ui_util_set_ui (priv->ui_component, 
 			       DATADIR, "gpdf-window-ui.xml", "GPDF",
 			       NULL);
+
+	bonobo_ui_component_add_listener(priv->ui_component, "ViewFullScreen",
+					 listener_ViewFullScreen, gpdf_window);
 }
 
 static void
@@ -773,6 +966,9 @@ GPdfWindow *gpdf_window_construct (GPdfWindow *gpdf_window)
 	gw_setup_recent_menu (gpdf_window, recent_facade, uri_in);
 	gw_setup_recent_toolitem (gpdf_window, recent_facade, uri_in);
 
+	g_signal_connect (G_OBJECT (gpdf_window), "window_state_event",
+			  G_CALLBACK (gpdf_window_window_state_changed), NULL);
+
 	return gpdf_window;
 }
 
