@@ -232,6 +232,9 @@ gpdf_persist_stream_create_password_dialog (GPdfPersistStream *ps,
     g_signal_connect ((gpointer) okbutton1, "clicked",
 		      G_CALLBACK (gpdf_persist_stream_on_okbutton_clicked),
 		      (gpointer)dialog1);
+    g_signal_connect ((gpointer) helpbutton1, "clicked",
+		      G_CALLBACK (gpdf_persist_stream_on_helpbutton_clicked),
+		      (gpointer)dialog1);
     return dialog1;
 }
 
