@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: gpdf-sidebar.c,v 1.3 2004/01/13 09:19:57 mkretzschmar Exp $
+ * $Id: gpdf-sidebar.c,v 1.6 2004/01/22 08:51:57 rcoscali Exp $
  */
 
 #include <string.h>
@@ -332,7 +332,7 @@ gpdf_sidebar_instance_init (GPdfSidebar *sidebar)
                             sidebar->priv->title_label,
                             FALSE, FALSE, 0);
        
-        arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_ETCHED_IN);
+        arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_NONE);
         gtk_widget_show (arrow);
         gtk_box_pack_end (GTK_BOX (button_hbox), arrow, FALSE, FALSE, 0);
         gtk_tooltips_set_tip (tooltips, arrow,
@@ -381,7 +381,7 @@ gpdf_sidebar_instance_init (GPdfSidebar *sidebar)
                             FALSE, FALSE, 0);
 
         /* And an arrow */
-        arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_ETCHED_IN);
+        arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_NONE);
         gtk_widget_show (arrow);
         gtk_box_pack_end (GTK_BOX (button_hbox), arrow, FALSE, FALSE, 0);
         gtk_tooltips_set_tip (tooltips, arrow,
@@ -448,7 +448,7 @@ gpdf_sidebar_instance_init (GPdfSidebar *sidebar)
 
         frame = gtk_frame_new (NULL);
         gtk_frame_set_shadow_type (GTK_FRAME (frame), 
-				   GTK_SHADOW_ETCHED_IN);
+				   GTK_SHADOW_NONE);
         gtk_widget_show (frame);
 	sidebar->priv->title_frame = frame;
  
@@ -473,6 +473,19 @@ gpdf_sidebar_new (void)
 	return GTK_WIDGET (g_object_new (GPDF_SIDEBAR_TYPE, NULL));
 }
 
+static void
+set_tools_button_sensitivity (GPdfSidebar *sidebar, GPdfSidebarPage *page)
+{
+    if (!sidebar->priv->current) return;
+
+    if (GTK_IS_WIDGET(sidebar->priv->tools_button) &&
+        !strncmp(page->id,
+                 sidebar->priv->current->id,
+                 strlen(sidebar->priv->current->id)))
+      gtk_widget_set_sensitive (GTK_WIDGET(sidebar->priv->tools_button),
+                                page->tools_menu && GTK_IS_MENU (page->tools_menu));
+}
+
 static void
 select_page (GPdfSidebar *sidebar, GPdfSidebarPage *page)
 {
@@ -484,10 +497,8 @@ select_page (GPdfSidebar *sidebar, GPdfSidebarPage *page)
 	gtk_widget_set_sensitive (GTK_WIDGET(sidebar->priv->remove_button),
 				  page->can_remove);
 #endif
-        
-        gtk_widget_set_sensitive (GTK_WIDGET(sidebar->priv->tools_button),
-                                  page->tools_menu && GTK_IS_MENU (page->tools_menu));                
-	
+        set_tools_button_sensitivity (sidebar, page);
+          
 	g_signal_emit (G_OBJECT (sidebar), sidebar_signals[PAGE_CHANGED], 0, 
 		       page->id);
 }
@@ -517,7 +528,7 @@ gpdf_sidebar_add_page	(GPdfSidebar *sidebar,
 	page->title = g_strdup (title);
         page->tools_menu = tools_menu; 
 	page->can_remove = can_remove;
-	
+
         item = gtk_menu_item_new_with_label (title); 
 	g_object_set_data (G_OBJECT(item), "page", (gpointer)page);
 	g_signal_connect (G_OBJECT(item), "activate",
@@ -639,5 +650,7 @@ gpdf_sidebar_set_page_tools_menu (GPdfSidebar *sidebar,
         old_menu = page->tools_menu;
         page->tools_menu = tools_menu;
 
+        set_tools_button_sensitivity (sidebar, page);
+        
         return old_menu; 
 }
